@@ -1,3 +1,4 @@
+#!/usr/bin/perl -w
 package XML::Bare;
 
 use Carp;
@@ -7,7 +8,7 @@ use utf8;
 require Exporter;
 require DynaLoader;
 @ISA = qw(Exporter DynaLoader);
-$VERSION = "0.52";
+$VERSION = "0.53";
 use vars qw($VERSION *AUTOLOAD);
 
 *AUTOLOAD = \&XML::Bare::AUTOLOAD;
@@ -22,7 +23,7 @@ XML::Bare - Minimal XML parser implemented via a C state engine
 
 =head1 VERSION
 
-0.52
+0.53
 
 =cut
 
@@ -282,15 +283,58 @@ sub new_node {
 }
 
 sub simplify {
-  my $self = shift;
-  my $root = shift;
-  my %ret;
-  foreach my $name ( keys %$root ) {
-    next if( $name =~ m|^_| || $name eq 'comment' || $name eq 'value' );
-    my $val = xval $root->{$name};
-    $ret{ $name } = $val;
-  }
-  return \%ret;
+    my $node = CORE::shift;
+    my $ref = ref( $node );
+    if( $ref eq 'ARRAY' ) {
+        my @ret;
+        for my $sub ( @$node ) {
+            CORE::push( @ret, simplify( $sub ) );
+        }
+        return \@ret;
+    }
+    if( $ref eq 'HASH' ) {
+        my %ret;
+        my $cnt = 0;
+        for my $key ( keys %$node ) {
+            next if( $key eq 'comment' || $key eq 'value' || $key =~ m/^_/ );
+            $cnt++;
+            $ret{ $key } = simplify( $node->{ $key } );
+        }
+        if( $cnt == 0 ) {
+            return $node->{'value'};
+        }
+        return \%ret;
+    }
+    return $node;
+}
+
+sub hash2xml {
+    my ( $node, $name ) = @_;
+    my $ref = ref( $node );
+    return '' if( $name && $name =~ m/^\_/ );
+    my $txt = $name ? "<$name>" : '';
+    if( $ref eq 'ARRAY' ) {
+       $txt = '';
+       for my $sub ( @$node ) {
+           $txt .= hash2xml( $sub, $name );
+       }
+       return $txt;
+    }
+    elsif( $ref eq 'HASH' ) {
+       for my $key ( keys %$node ) {
+           $txt .= hash2xml( $node->{ $key }, $key );
+       }
+    }
+    else {
+        $node ||= '';
+        if( $node =~ /[<]/ ) { $txt .= '<![CDATA[' . $node . ']]>'; }
+        else { $txt .= $node; }
+    }
+    if( $name ) {
+        $txt .= "</$name>";
+    }
+        
+    return $txt;
 }
 
 # Save an XML hash tree into a file
@@ -1389,6 +1433,13 @@ Take a node with children that have immediate values and
 creates a hashref to reference those values by the name of
 each child.
 
+=item * C<< XML::Bare::hash2xml( [hashref] ) >>
+
+Take a recursive hash tree ( perhaps generated by the simplify function ) and turn it
+into a raw XML string. Note that this function does not indent nicely. You will need
+to feed this string back into the parser and output it again if you want it to look
+nice. ( or you could use the 'clean' function to do it in one go )
+
 =item * C<< XML::Bare->new( text => "[xml]", unsafe => 1 ) >>
 
 An extra speedy way to parse XML. It is unsafe; may harm pets and children. Don't
